<?php

namespace Tests\Feature;

use Illuminate\Foundation\Http\MaintenanceModeBypassCookie;
use Illuminate\Foundation\Http\Middleware\PreventRequestsDuringMaintenance;
use Illuminate\Foundation\Exceptions\RegisterErrorViewPaths;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Artisan;
use Symfony\Component\HttpFoundation\Cookie;
use Tests\TestCase;

class MaintenanceModeSampleTest extends TestCase
{
    public function tearDown(): void
    {
        @unlink(storage_path('framework/down'));
    }

    public function _testBasicMaintenanceModeResponse()
    {
        Artisan::call('up');
        Artisan::call(
            'down',
            [
                 '--retry' => 60
            ]
        );

        Route::get('/foo', function () {
            return 'Hello World';
        })->middleware(PreventRequestsDuringMaintenance::class);

        $response = $this->get('/foo');

        $response->assertStatus(503);
        $response->assertHeader('Retry-After', '60');
    }

    public function testMaintenanceModeCanRedirectWithBypassCookie()
    {
        $this->_testBasicMaintenanceModeResponse();
        Artisan::call('up');
        file_put_contents(
            resource_path('views/quera.blade.php'),
            'This is a test from Quera.'
        );
        Artisan::call(
            'down',
            [
                 '--retry' => 60,
                 '--secret' => 'foo',
                 '--time' => 24,
                 '--render' => 'quera'
            ]
        );

        Route::get('/foo', function () {
            return 'Hello World';
        })->middleware(PreventRequestsDuringMaintenance::class);

        $response = $this->get('/foo');

        $response->assertStatus(302);
        $response->assertCookie('laravel_maintenance');
        $cookie = $this->getCookie($response, 'laravel_maintenance');

        Carbon::setTestNow(now()->addSeconds(12 * 3600 + 1));

        Route::get('/test', function () {
            return 'Hello World';
        })->middleware(PreventRequestsDuringMaintenance::class);

        $response = $this->withUnencryptedCookies([
            'laravel_maintenance' => $cookie->getValue(),
        ])->get('/test');

        $response->assertStatus(200);

        Carbon::setTestNow();

        Carbon::setTestNow(now()->addSeconds(24 * 3600 + 1));

        Route::get('/test', function () {
            return 'Hello World';
        })->middleware(PreventRequestsDuringMaintenance::class);

        $response = $this->withUnencryptedCookies([
            'laravel_maintenance' => $cookie->getValue(),
        ])->get('/test');

        $response->assertStatus(503);
    }

    protected function getCookie($response, $cookieName, $isEncrypted = false)
    {
        foreach ($response->headers->getCookies() as $cookie) {
            if ($cookie->getName() === $cookieName) {
                return $cookie;
            }
        }
    }
}
